"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThingsCloudClient = void 0;
var client_1 = require("@c8y/client");
var ThingsCloudClient = /** @class */ (function () {
    /**
     * Things Cloudのデータを取得する
     * @class
     * @param {FetchClient} deviceId deviceId or groupId
     *
     *
     */
    function ThingsCloudClient(fetchClient) {
        this.fetchClient = fetchClient;
        this.measurementService = new client_1.MeasurementService(fetchClient);
        this.alarmService = new client_1.AlarmService(fetchClient);
        this.eventService = new client_1.EventService(fetchClient);
        this.inventoryService = new client_1.InventoryService(fetchClient);
        this.realtime = new client_1.Realtime(fetchClient);
    }
    /**
     * Things Cloud に接続し、Alarm 一覧を取得する
     *
     * @param {object} param
     *
     * @returns Promise<IResultList<IAlarm>>
     *
     * @example
     * ```typescript
     *
     *     const alarmsFilter = {
     *        deviceId: "123456",
     *        filter: {
     *          pageSize: 10,
     *          type: "c8y_UnavailabilityAlarm"
     *        }
     *     };
     *
     *    (async () => {
     *       const alarms = await thingsCloudClient.getAlarms(alarmsFilter);
     *    })();
     * ```
     */
    ThingsCloudClient.prototype.getAlarms = function (param) {
        return __awaiter(this, void 0, void 0, function () {
            var requestFilter;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        requestFilter = (param.filter) ? JSON.parse(JSON.stringify(param.filter)) : {};
                        if (param.deviceId) {
                            requestFilter.source = param.deviceId;
                        }
                        requestFilter.withTotalPages = true;
                        return [4 /*yield*/, this.alarmService.list(requestFilter)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Things Cloud に接続し、Event 一覧を取得する
     *
     * @param {object} param
     *
     * @returns Promise<IResultList<IEvent>>
     *
     * @example
     * ```typescript
     *
     *     const eventsFilter = {
     *        deviceId: "123456",
     *        filter: {
     *          pageSize: 10,
     *          type: "c8y_LocationUpdate"
     *        }
     *     };
     *
     *    (async () => {
     *       const events = await thingsCloudClient.getEvents(eventsFilter);
     *    })();
     * ```
     */
    ThingsCloudClient.prototype.getEvents = function (param) {
        return __awaiter(this, void 0, void 0, function () {
            var requestFilter;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        requestFilter = (param.filter) ? JSON.parse(JSON.stringify(param.filter)) : {};
                        if (param.deviceId) {
                            requestFilter.source = param.deviceId;
                        }
                        requestFilter.withTotalPages = true;
                        return [4 /*yield*/, this.eventService.list(requestFilter)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Things Cloud に接続し、Measurement 一覧を取得する
     *
     * @param {object} param
     *
     * @returns Promise<IResultList<IMeasurement>>
     *
     * @example
     * ```typescript
     *
     *     const measurementsFilter = {
     *        deviceId: "123456",
     *        filter: {
     *          pageSize: 10,
     *          valueFragmentType: "c8y_Temperature"
     *        }
     *     };
     *
     *    (async () => {
     *       const measurements = await thingsCloudClient.getMeasurements(measurementsFilter);
     *    })();
     * ```
     */
    ThingsCloudClient.prototype.getMeasurements = function (param) {
        return __awaiter(this, void 0, void 0, function () {
            var requestFilter;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        requestFilter = (param.filter) ? JSON.parse(JSON.stringify(param.filter)) : {};
                        if (param.deviceId) {
                            requestFilter.source = param.deviceId;
                        }
                        return [4 /*yield*/, this.measurementService.list(requestFilter)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * MO ID に対し、サブアセット（ManagedObject の ID）を取得する。
     * ID がデバイスの場合、単独のデバイスが設定され、アセットの場合、再帰的に
     * 子アセットのデバイスが設定される。
     *
     * @param {string} moId MO ID
     *
     * @returns Promise デバイスIDの配列
     *
     * @example
     * ```typescript
     *
     *     const moId = "123456";
     *
     *    (async () => {
     *       const moIds = await thingsCloudClient.getSubAssets(moId);
     *    })();
     * ```
     */
    ThingsCloudClient.prototype.getSubAssets = function (moId) {
        return __awaiter(this, void 0, void 0, function () {
            var mos;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getSubAssetMOs(moId)];
                    case 1:
                        mos = _a.sent();
                        return [2 /*return*/, mos.map(function (mo) { return mo.id; })];
                }
            });
        });
    };
    /**
     * MO ID に対し、サブアセット（ManagedObject）を取得する。
     * ID がデバイスの場合、単独のデバイスが設定され、アセットの場合、再帰的に
     * 子アセットのデバイスが設定される。
     *
     * @param {string} moId MO ID
     *
     * @returns Promise デバイス ManagedObject の配列
     *
     * @example
     * ```typescript
     *
     *     const moId = "123456";
     *
     *    (async () => {
     *       const mos = await thingsCloudClient.getSubAssetMOs(moId);
     *    })();
     * ```
     */
    ThingsCloudClient.prototype.getSubAssetMOs = function (moId) {
        return __awaiter(this, void 0, void 0, function () {
            var devices;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        devices = [];
                        return [4 /*yield*/, this.getSubAssetMOsImpl(moId, devices)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, devices];
                }
            });
        });
    };
    /**
     * リアルタイムにイベント等を取得するための Realtime オブジェクトを取得する
     *
     * @returns Realtime のインスタンス
     */
    ThingsCloudClient.prototype.getRealtime = function () {
        return this.realtime;
    };
    /**
     * リアルタイム通知のチャネル監視を解除する
     *
     * @param subscription チャネル監視中のサブスクリプション
     */
    ThingsCloudClient.prototype.unsubscribeRealtime = function (subscription) {
        this.realtime.unsubscribe(subscription);
    };
    /**
     * デバイスかグループを判断し、デバイスの場合はdevicesへpush、
     * グループの場合は再帰的に関数を呼び出す。
     *
     * @param {string} moId moId
     * @param {IManagedObject} devices デバイス情報のリスト
     *
     * @returns Promise<void>
     */
    ThingsCloudClient.prototype.getSubAssetMOsImpl = function (moId, devices) {
        return __awaiter(this, void 0, void 0, function () {
            var res, _i, _a, data;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.inventoryService.detail(moId)];
                    case 1:
                        res = _b.sent();
                        if (!(res.data.childAssets.references.length > 0)) return [3 /*break*/, 6];
                        _i = 0, _a = res.data.childAssets.references;
                        _b.label = 2;
                    case 2:
                        if (!(_i < _a.length)) return [3 /*break*/, 5];
                        data = _a[_i];
                        return [4 /*yield*/, this.getSubAssetMOsImpl(data.managedObject.id, devices)];
                    case 3:
                        _b.sent(); // 再帰
                        _b.label = 4;
                    case 4:
                        _i++;
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 7];
                    case 6:
                        // 指定されたのはデバイス
                        if (devices.map(function (mo) { return mo.id; }).indexOf(res.data.id) > -1)
                            return [2 /*return*/];
                        devices.push(res.data);
                        _b.label = 7;
                    case 7: return [2 /*return*/];
                }
            });
        });
    };
    return ThingsCloudClient;
}());
exports.ThingsCloudClient = ThingsCloudClient;
