"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThingsCloudDataSupplier = void 0;
var rxjs_1 = require("rxjs");
var thingsCloudClient_1 = require("./thingsCloudClient");
var ThingsCloudDataSupplier = /** @class */ (function () {
    /**
     * ThingsCloudDataSupplierクラスのコンストラクタです。
     * 各メソッドでThingsCloudの対象テナントからデータを取得し、そのデータをフラット化して返却します。
     *
     * @class
     *
     * @param {baseUrl} baseUrl 情報を取得するテナントのURL
     * @param {user} user 情報を取得するための認証情報(ユーザ名)
     * @param {password} password 情報を取得するための認証情報(パスワード)
     *
     * @desc
     * *フラット化について*
     * このクラスでは、各メソッドで結果がフラット化され返却されます。例えば、データを取得した結果が
     * ```json
     * [
     *   {
     *     "c8y_Temperature": {
     *       "T": {
     *         "value": 18.0,
     *         "unit": "C"
     *       }
     *     }
     *   },
     *   :
     * ]
     * ```
     * のような JSON 構造だった場合、
     * ```json
     * [
     *   {
     *     "c8y_Temperature_T_value": 18.0,
     *     "c8y_Temperature_T_unit": "C"
     *   },
     *   :
     * ]
     * ```
     * のような形式の変換がなされます。
     * このようなデータ形式の変換操作をフラット化と呼びます。
     */
    function ThingsCloudDataSupplier(fetchClient) {
        this.thingsCloudClient = new thingsCloudClient_1.ThingsCloudClient(fetchClient);
    }
    /**
     * deviceId, measurementsFilterで指定されたデバイスのMeasurement情報を
     * 取得し、フラット化された(階層的なオブジェクト構造があった場合、ルートに設定した)
     * オブジェクトの配列として返却します。
     *
     * @param {string} deviceId deviceId or groupId (groupId を指定した場合、再帰的に子のデバイスが検索され、対象となります。)
     * @param {object} measurementsFilter Measurement のフィルタ条件を指定する情報
     *
     * @returns Measurement配列で解決する Promise
     * @example
     * ```typescript
     * async () => {
     *   :
     *   // thingsCloudDataSupplier にインスタンスを格納
     *   :
     *   const measurements = await thingsCloudDataSupplier.getMeasurementsArray('123456', {
     *     type: 'c8y_TemperatureMeasurement',
     *     dateFrom: '2022-02-01T00:00:00.000+09:00',
     *     dateTo: '2022-02-28T23:59:59.999+09:00'
     *   });
     * }
     * ```
     */
    ThingsCloudDataSupplier.prototype.getMeasurementsArray = function (deviceId, measurementsFilter) {
        return __awaiter(this, void 0, void 0, function () {
            var flattenOption, deviceIds, measurements, promises, measurementsList;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        flattenOption = measurementsFilter.flatten;
                        delete measurementsFilter.flatten;
                        return [4 /*yield*/, this.thingsCloudClient.getSubAssets(deviceId)];
                    case 1:
                        deviceIds = _a.sent();
                        measurements = [];
                        promises = deviceIds.map(function (deviceId) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, this.thingsCloudClient.getMeasurements({ deviceId: deviceId, filter: measurementsFilter })];
                                    case 1: return [2 /*return*/, (_a.sent()).data];
                                }
                            });
                        }); });
                        return [4 /*yield*/, Promise.all(promises)];
                    case 2:
                        measurementsList = _a.sent();
                        measurementsList.forEach(function (result) {
                            measurements = measurements.concat(result);
                        });
                        if (flattenOption || flattenOption === void 0)
                            measurements = this.flatten(measurements);
                        measurements.forEach(function (f) { return f.time = new Date(f.time); });
                        return [2 /*return*/, measurements];
                }
            });
        });
    };
    /**
     * deviceId, alarmsFilterで指定されたデバイスのAlarm情報を
     * 取得し、フラット化された(階層的なオブジェクト構造があった場合、ルートに設定した)
     * オブジェクトの配列として返却します。
     *
     * @param {string} deviceId deviceId or groupId (groupId を指定した場合、再帰的に子のデバイスが検索され、対象となります。)
     * @param {object} alarmsFilter Alarm のフィルタ条件を指定する情報
     *
     * @returns Alarm配列で解決する Promise
     * @example
     * ```typescript
     * async () => {
     *   :
     *   // thingsCloudDataSupplier にインスタンスを格納
     *   :
     *   const alarms = await thingsCloudDataSupplier.getAlarmsArray('123456', {
     *     type: 'c8y_UnavailabilityAlarm',
     *     dateFrom: '2022-02-01T00:00:00.000+09:00',
     *     dateTo: '2022-02-28T23:59:59.999+09:00',
     *     status: 'CLEARED'
     *   });
     * }
     * ```
     */
    ThingsCloudDataSupplier.prototype.getAlarmsArray = function (deviceId, alarmsFilter) {
        return __awaiter(this, void 0, void 0, function () {
            var flattenOption, deviceIds, alarms, promises, alarmsList;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        flattenOption = alarmsFilter.flatten;
                        delete alarmsFilter.flatten;
                        return [4 /*yield*/, this.thingsCloudClient.getSubAssets(deviceId)];
                    case 1:
                        deviceIds = _a.sent();
                        alarms = [];
                        promises = deviceIds.map(function (deviceId) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, this.thingsCloudClient.getAlarms({ deviceId: deviceId, filter: alarmsFilter })];
                                    case 1: return [2 /*return*/, (_a.sent()).data];
                                }
                            });
                        }); });
                        return [4 /*yield*/, Promise.all(promises)];
                    case 2:
                        alarmsList = _a.sent();
                        alarmsList.forEach(function (result) {
                            alarms = alarms.concat(result);
                        });
                        if (flattenOption || flattenOption === void 0)
                            alarms = this.flatten(alarms);
                        // alarms.forEach(f => f.time = new Date(f.time)); IAlarm の time は string 
                        return [2 /*return*/, alarms];
                }
            });
        });
    };
    /**
     * deviceId, eventsFilterで指定されたデバイスのEvent情報を
     * 取得し、フラット化された(階層的なオブジェクト構造があった場合、ルートに設定した)
     * オブジェクトの配列として返却します。
     *
     * @param {string} deviceId deviceId or groupId (groupId を指定した場合、再帰的に子のデバイスが検索され、対象となります。)
     * @param {object} eventsFilter Event のフィルタ条件を指定する情報
     *
     * @returns Event配列で解決する Promise
     * @example
     * ```typescript
     * async () => {
     *   :
     *   // thingsCloudDataSupplier にインスタンスを格納
     *   :
     *   const events = await thingsCloudDataSupplier.getEventsArray('123456', {
     *     type: 'c8y_UpdateEvent',
     *     dateFrom: '2022-02-01T00:00:00.000+09:00',
     *     dateTo: '2022-02-28T23:59:59.999+09:00'
     *   });
     * }
     * ```
     */
    ThingsCloudDataSupplier.prototype.getEventsArray = function (deviceId, eventsFilter) {
        return __awaiter(this, void 0, void 0, function () {
            var flattenOption, deviceIds, events, promises, eventsList;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        flattenOption = eventsFilter.flatten;
                        delete eventsFilter.flatten;
                        return [4 /*yield*/, this.thingsCloudClient.getSubAssets(deviceId)];
                    case 1:
                        deviceIds = _a.sent();
                        events = [];
                        promises = deviceIds.map(function (deviceId) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, this.thingsCloudClient.getEvents({ deviceId: deviceId, filter: eventsFilter })];
                                    case 1: return [2 /*return*/, (_a.sent()).data];
                                }
                            });
                        }); });
                        return [4 /*yield*/, Promise.all(promises)];
                    case 2:
                        eventsList = _a.sent();
                        eventsList.forEach(function (result) {
                            events = events.concat(result);
                        });
                        if (flattenOption || flattenOption === void 0)
                            events = this.flatten(events);
                        // events.forEach(f => f.time = new Date(f.time)); IEvent の time は string
                        return [2 /*return*/, events];
                }
            });
        });
    };
    /**
     * deviceIdで指定されたデバイスのMOリストをオブジェクトの配列として返却します
     *
     * @param {string} deviceId deviceId or groupId (groupId を指定した場合、再帰的に子のデバイスが検索され、対象となります。)
     *
     * @returns IManagedObject配列で解決する Promise
     * @example
     * ```typescript
     *
     * ```
     */
    ThingsCloudDataSupplier.prototype.getDevicesArray = function (deviceId) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.thingsCloudClient.getSubAssetMOs(deviceId)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * 対象のオブジェクトを監視し、メジャーメントのリアルタイム通知を受け取ります
     *
     * @param {string} target 監視対象オブジェクト deviceId or '*' (テナント内全てのデバイスが対象)
     *
     * @returns リアルタイム通知で受け取ったメジャーメント
     * @example
     * ```typescript
     *    thingsCloudDataSupplier.getRealtimeMeasurement('12345')
     *      .subscribe(notifiedData => {
                updateData();
     *      })
     * ```
     */
    ThingsCloudDataSupplier.prototype.getRealtimeMeasurement = function (target) {
        this.realtimeSubscription = this.thingsCloudClient.getRealtime();
        var notified = new rxjs_1.Subject();
        this.realtimeSubscription.subscribe("/measurements/".concat(target), function (notifiedData) {
            notified.next(notifiedData.data.data);
        });
        return notified;
    };
    /**
     * 対象のオブジェクトを監視し、アラームのリアルタイム通知を受け取ります
     *
     * @param {string} target 監視対象オブジェクト deviceId or '*' (テナント内全てのデバイスが対象)
     *
     * @returns リアルタイム通知で受け取ったアラーム
     * @example
     * ```typescript
     *    thingsCloudDataSupplier.getRealtimeAlarm('12345')
     *      .subscribe(notifiedData => {
     *         updateData();
     *       })
     *  ```
     */
    ThingsCloudDataSupplier.prototype.getRealtimeAlarm = function (target) {
        this.realtimeSubscription = this.thingsCloudClient.getRealtime();
        var notified = new rxjs_1.Subject();
        this.realtimeSubscription.subscribe("/alarms/".concat(target), function (notifiedData) {
            notified.next(notifiedData.data.data);
        });
        return notified;
    };
    /**
     * 対象のオブジェクトを監視し、イベントのリアルタイム通知を受け取ります
     *
     * @param {string} target 監視対象オブジェクト deviceId or '*' (テナント内全てのデバイスが対象)
     *
     * @returns リアルタイム通知で受け取ったイベント
     * @example
     * ```typescript
     *    thingsCloudDataSupplier.getRealtimeEvent('12345')
     *      .subscribe(notifiedData => {
     *        updateData();
     *      })
     * ```
     */
    ThingsCloudDataSupplier.prototype.getRealtimeEvent = function (target) {
        this.realtimeSubscription = this.thingsCloudClient.getRealtime();
        var notified = new rxjs_1.Subject();
        this.realtimeSubscription.subscribe("/events/".concat(target), function (notifiedData) {
            notified.next(notifiedData.data.data);
        });
        return notified;
    };
    /**
     * 対象のオブジェクトを監視し、マネージドオブジェクトのリアルタイム通知を受け取ります
     *
     * @param {string} target 監視対象オブジェクト deviceId or '*' (テナント内全てのデバイスが対象)
     *
     * @returns リアルタイム通知で受け取ったマネージドオブジェクト
     * @example
     * ```typescript
     *    thingsCloudDataSupplier.getRealtimeManagedObject('12345')
     *      .subscribe(notifiedData => {
                updateData();
     *      })
     * ```
     */
    ThingsCloudDataSupplier.prototype.getRealtimeManagedObject = function (target) {
        this.realtimeSubscription = this.thingsCloudClient.getRealtime();
        var notified = new rxjs_1.Subject();
        this.realtimeSubscription.subscribe("/managedobjects/".concat(target), function (notifiedData) {
            notified.next(notifiedData.data.data);
        });
        return notified;
    };
    /**
     * リアルタイム通知のチャネル監視を解除します
     */
    ThingsCloudDataSupplier.prototype.unsubscribeRealtime = function () {
        this.thingsCloudClient.unsubscribeRealtime(this.realtimeSubscription);
    };
    /**
     * 指定された object の配列に関し、各 object の階層を平坦化し 1 層に変換します。
     * 例： [{a: 1, b: {c: 5}}] ---> [{a: 1, b_c: 5}]
     * object 値に配列を含む場合、配列内の object は平坦化されないことに注意して下さい。
     * 例： [{a: 1, b: [{c: 5}]}] ---> [{a: 1, b: [{c: 5}]}] (変更なし)
     *
     * @param objectArray object の配列
     * @returns  各要素が平坦化(つなぎ文字は _)された object の配列(Plot で利用可能な形式)
     * @example
     * ```typescript
     * :
     * // thingsCloudDataSupplier にインスタンスを格納
     * :
     * const array = [{a: 1, b: {c: 5}}];
     * const result = thingsCloudDataSupplier.flatten(array);
     * console.log(result); // expected: [{a: 1, b_c: 5}]
     * ```
     */
    ThingsCloudDataSupplier.prototype.flatten = function (objectArray) {
        var _this = this;
        objectArray.forEach(function (obj) { return _this.flattenImpl(obj, '', obj); });
        return objectArray;
    };
    /**
     * flatten の実処理を行います。parent のデータが平坦化され、上書きされます。
     *
     * @param parent flatten 先のデータを格納する object
     * @param parentKey 階層化されたキー情報を連結したもの
     * @param obj flatten 対象となる object (再帰により子オブジェクトが指定される)
     * @example
     * ```typescript
     * :
     * // thingsCloudDataSupplier にインスタンスを格納
     * :
     * const obj = {a: 1, b: {c: 5}};
     * const result = thingsCloudDataSupplier._flatten(obj, '', obj);
     * console.log(result); // expected: {a: 1, b_c: 5}
     * ```
     */
    ThingsCloudDataSupplier.prototype.flattenImpl = function (parent, parentKey, obj) {
        var _this = this;
        var keys = Object.keys(obj);
        if (keys.length == 0)
            return;
        if (parentKey)
            delete parent[parentKey];
        keys.forEach(function (key) {
            var v = obj[key];
            var ckey = parentKey + ((parentKey !== '') ? '_' : '') + key;
            if (v !== null && typeof v === 'object' && !Array.isArray(v)) {
                _this.flattenImpl(parent, ckey, v);
            }
            else {
                if (parent !== obj) {
                    parent[ckey] = v;
                }
            }
        });
    };
    return ThingsCloudDataSupplier;
}());
exports.ThingsCloudDataSupplier = ThingsCloudDataSupplier;
